'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _Config = require('./Config');

var _Config2 = _interopRequireDefault(_Config);

var UNIT_TYPE_CHARACTERS = {
  archer: 'a',
  captive: 'C',
  sludge: 's',
  thickSludge: 'S',
  warrior: '@',
  wizard: 'w'
};

var UNIT_TYPE_STYLES = {
  archer: _chalk2['default'].yellow,
  captive: _chalk2['default'].magenta,
  sludge: _chalk2['default'].green,
  thickSludge: _chalk2['default'].green,
  warrior: _chalk2['default'].cyan,
  wizard: _chalk2['default'].blue
};

var UI = (function () {
  function UI() {
    _classCallCheck(this, UI);
  }

  _createClass(UI, null, [{
    key: 'print',
    value: function print(message) {
      if (_Config2['default'].outStream) {
        _Config2['default'].outStream.write(message);
      }
    }
  }, {
    key: 'printLine',
    value: function printLine(message) {
      UI.print(message + '\n');
    }
  }, {
    key: 'printLineWithDelay',
    value: function printLineWithDelay(message) {
      return new _bluebird2['default'](function (resolve) {
        UI.printLine(message);
        setTimeout(resolve, _Config2['default'].delay * 1000);
      });
    }

    /*
     * Inquire
     */

  }, {
    key: 'request',
    value: function request(message) {
      return new _bluebird2['default'](function (resolve) {
        var name = 'request';
        _inquirer2['default'].prompt([{
          name: name,
          message: message,
          type: 'input'
        }], function (answers) {
          resolve(answers[name]);
        });
      });
    }
  }, {
    key: 'ask',
    value: function ask(message) {
      var defaultAnswer = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      return new _bluebird2['default'](function (resolve) {
        var name = 'confirmation';
        _inquirer2['default'].prompt([{
          name: name,
          message: message,
          type: 'confirm',
          'default': defaultAnswer
        }], function (answers) {
          resolve(answers[name]);
        });
      });
    }
  }, {
    key: 'choose',
    value: function choose(itemName, items) {
      return new _bluebird2['default'](function (resolve) {
        var choices = items.map(function (item) {
          return {
            name: item.toString(),
            value: item
          };
        });

        var name = 'choice';
        _inquirer2['default'].prompt([{
          name: name,
          choices: choices,
          type: 'list',
          message: 'Choose ' + itemName + ':'
        }], function (answers) {
          resolve(answers[name]);
        });
      });
    }

    /*
     * Graphics
     */

  }, {
    key: 'printTrace',
    value: function printTrace(trace) {
      var turnNumber = 1;
      return _bluebird2['default'].reduce(trace, function (_, turn) {
        UI.printLine('-------------------------- turn ' + turnNumber + ' --------------------------');
        turnNumber += 1;

        var floor = turn.floor;
        var log = turn.log;

        return UI.printFloor(floor).then(function () {
          return UI.printLog(log);
        });
      }, null);
    }
  }, {
    key: 'printFloor',
    value: function printFloor(floor) {
      var floorCharacter = UI.getFloorCharacter(floor.width, floor.height, floor.stairsLocation, floor.units);
      return UI.printLineWithDelay(floorCharacter);
    }
  }, {
    key: 'printLog',
    value: function printLog(log) {
      return _bluebird2['default'].reduce(log, function (_, entry) {
        var unitType = entry.unitType;
        var message = entry.message;

        return UI.printLineWithDelay(UI.getUnitStyle(unitType)(message));
      }, null);
    }
  }, {
    key: 'getFloorCharacter',
    value: function getFloorCharacter(width, height, stairsLocation, units) {
      var styled = arguments.length <= 4 || arguments[4] === undefined ? true : arguments[4];

      var rows = [];
      rows.push('╔' + '═'.repeat(width) + '╗');

      var _loop = function (y) {
        var row = '║';

        var _loop2 = function (x) {
          var foundUnit = units.find(function (unit) {
            return unit.position.x === x && unit.position.y === y;
          }); // eslint-disable-line no-loop-func
          if (foundUnit) {
            row += styled ? UI.getUnitStyle(foundUnit.type)(UI.getUnitCharacter(foundUnit.type)) : UI.getUnitCharacter(foundUnit.type);
          } else if (stairsLocation[0] === x && stairsLocation[1] === y) {
            row += '>';
          } else {
            row += ' ';
          }
        };

        for (var x = 0; x < width; x++) {
          _loop2(x);
        }

        row += '║';
        rows.push(row);
      };

      for (var y = 0; y < height; y++) {
        _loop(y);
      }

      rows.push('╚' + '═'.repeat(width) + '╝');
      return rows.join('\n');
    }
  }, {
    key: 'getUnitCharacter',
    value: function getUnitCharacter(type) {
      return UNIT_TYPE_CHARACTERS[type];
    }
  }, {
    key: 'getUnitStyle',
    value: function getUnitStyle(type) {
      return UNIT_TYPE_STYLES[type];
    }
  }]);

  return UI;
})();

exports['default'] = UI;
module.exports = exports['default'];